/*
 * Jitsi, the OpenSource Java VoIP and Instant Messaging client.
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jitsi.util.asm;

import org.apache.tools.ant.*;
import org.apache.tools.ant.types.*;
import org.objectweb.asm.*;
import org.objectweb.asm.commons.*;

import java.io.*;
import java.util.*;

/**
 * Renames packages in class files.
 *
 * @author Damian Minkov
 */
public class PackageRenamer
    extends Task
{
    private ArrayList<FileSet> filesets = new ArrayList<FileSet>();
    private ArrayList<Rule> rules = new ArrayList<Rule>();

    /**
     * Called by the project to let the task do its work.
     *
     * @exception BuildException if something goes wrong with the build
     */
    public void execute()
        throws BuildException
    {
        for(FileSet fileSet : filesets)
        {
            File dir = fileSet.getDir();
            if(dir.isDirectory())
                processDir(dir);
            else
                getProject().log("Configured dir is not an actual folder!",
                    Project.MSG_WARN);
        }
    }

    /**
     * Process dirs.
     * @param dir the dir to process.
     */
    private void processDir(File dir)
    {
        for(File f : dir.listFiles())
        {
            if(f.isDirectory())
                processDir(f);
            else
                processFile(f);
        }
    }

    /**
     * Process a file by renaming the packages
     * by the rules specified.
     * @param file the file to process
     */
    private void processFile(File file)
    {
        try
        {
            if(!file.getName().endsWith(".class"))
                return;

            ByteArrayOutputStream out = new ByteArrayOutputStream();
            copy(new FileInputStream(file), out);
            byte[] remappedBytecode = rewriteDefaultPackageClassNames(
                out.toByteArray());
            FileOutputStream outFile = new FileOutputStream(file);
            outFile.write(remappedBytecode);
            outFile.flush();
            outFile.close();
        } catch (IOException e)
        {
            throw new RuntimeException("Could not rewrite class " + file);
        }
    }

    /**
     * Rewrites the package name, does the real job.
     * @param bytecode
     * @return
     * @throws IOException
     */
    public byte[] rewriteDefaultPackageClassNames(byte[] bytecode)
        throws IOException
    {
        ClassReader classReader = new ClassReader(bytecode);
        ClassWriter classWriter = new ClassWriter(classReader, 0);

        Remapper remapper = new PackageRemapper(rules, false);
        classReader.accept(
            new RemappingClassAdapter(classWriter, remapper),
            ClassReader.EXPAND_FRAMES);

        return classWriter.toByteArray();
    }

    /**
     * Adds filesets from ant build.xml. For now only dirs are processed.
     * @param fileset
     */
    public void addFileset(FileSet fileset)
    {
        filesets.add(fileset);
    }

    /**
     * Adds the rules to rewrite.
     * @param rule
     */
    public void addConfiguredRule(Rule rule)
    {
        if (rule.getPattern() == null || rule.getResult() == null)
            throw new IllegalArgumentException("The <rule> element requires both \"pattern\" and \"result\" attributes.");
        rules.add(rule);
    }

    /**
     * Copy a file and close the input.
     * @param is
     * @param out
     * @throws IOException
     */
    public static void copy(InputStream is, OutputStream out)
        throws IOException
    {
        byte[] buf = new byte[1024*4];
        for (;;)
        {
            int amt = is.read(buf);
            if (amt < 0)
                break;
            out.write(buf, 0, amt);
        }

        is.close();
    }
}
